"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateOrAdd = exports.publishAssessment = void 0;
const fs_1 = __importDefault(require("fs"));
const bentWrapper_1 = __importDefault(require("./bentWrapper"));
const path_1 = __importDefault(require("path"));
const config_1 = __importDefault(require("./config"));
const assessmentsTypes_1 = require("./assessmentsTypes");
const form_data_1 = __importDefault(require("form-data"));
const tools_1 = __importStar(require("./tools"));
const lodash_1 = __importDefault(require("lodash"));
const glob_promise_1 = __importDefault(require("glob-promise"));
const getJson = (0, bentWrapper_1.default)();
const ASSESSMENTS_DIR = '.guides/assessments';
const ROOT_METADATA = '.guides/content/index.json';
function listLibraries() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const res = yield getJson(`${(0, tools_1.getApiV1Url)()}/assessment_library`, undefined, (0, tools_1.getBearer)());
            const libraries = [];
            for (const _ of res) {
                libraries.push({
                    name: _.name,
                    id: _.id,
                    createdBy: _.createdBy
                });
            }
            return libraries;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
function updateTags(tags, id, hash) {
    const idTag = lodash_1.default.find(tags, { name: assessmentsTypes_1.API_ID_TAG });
    if (lodash_1.default.isUndefined(idTag)) {
        tags.push({
            name: assessmentsTypes_1.API_ID_TAG,
            value: id
        });
    }
    else {
        idTag.value = id;
    }
    const hashTag = lodash_1.default.find(tags, { name: assessmentsTypes_1.API_HASH_TAG });
    if (lodash_1.default.isUndefined(hashTag)) {
        tags.push({
            name: assessmentsTypes_1.API_HASH_TAG,
            value: hash
        });
    }
    else {
        hashTag.value = hash;
    }
}
function updateJSON(assessment, base) {
    return __awaiter(this, void 0, void 0, function* () {
        const working = path_1.default.join(base, ASSESSMENTS_DIR);
        const assessmentJsonFiles = yield (0, glob_promise_1.default)('*.json', { cwd: working, nodir: true });
        for (const file of assessmentJsonFiles) {
            const filePath = path_1.default.join(working, file);
            const assessmentString = yield fs_1.default.promises.readFile(filePath, { encoding: 'utf8' });
            const assessmentData = JSON.parse(assessmentString);
            if (assessmentData.taskId === assessment.taskId) {
                const hash = assessment.getHash();
                const id = assessment.getId();
                updateTags(assessmentData.source.metadata.tags, id, hash);
                yield fs_1.default.promises.writeFile(filePath, JSON.stringify(assessmentData, undefined, ' '));
                break;
            }
        }
    });
}
function publishAssessment(libraryId, assessment, isNew, base) {
    return __awaiter(this, void 0, void 0, function* () {
        libraryId = yield getLibraryId(libraryId);
        return _publishAssessment(libraryId, assessment, isNew, base);
    });
}
exports.publishAssessment = publishAssessment;
function _publishAssessment(libraryId, assessment, isNew, base) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), isNew ? 'POST' : 'PUT', 200);
            const postData = new form_data_1.default();
            postData.append('assessment', assessment.export());
            const archivePath = yield assessment.getBundle(base);
            if (!lodash_1.default.isUndefined(archivePath)) {
                const { size } = yield fs_1.default.promises.stat(archivePath);
                if (size === 0) {
                    console.log(`empty bundle`);
                }
                else {
                    postData.append('bundle', fs_1.default.createReadStream(archivePath), {
                        knownLength: size
                    });
                }
            }
            const headers = Object.assign(postData.getHeaders(), (0, tools_1.getBearer)());
            headers['Content-Length'] = yield new Promise(resolve => postData.getLength((_, length) => resolve(length)));
            const assessmentId = isNew ? '' : `/${assessment.assessmentId}`;
            yield updateJSON(assessment, base);
            yield api(`/assessment_library/${libraryId}/assessment${assessmentId}`, postData, headers);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                console.log(message);
                throw new Error(message);
            }
            console.log(error);
            throw error;
        }
    });
}
//return id if name passed
function getLibraryId(name) {
    return __awaiter(this, void 0, void 0, function* () {
        const libraries = yield listLibraries();
        const item = lodash_1.default.find(libraries, { name });
        if (lodash_1.default.isUndefined(item)) {
            return name;
        }
        return item.id;
    });
}
function updateOrAdd(libraryId, assessment, base) {
    return __awaiter(this, void 0, void 0, function* () {
        libraryId = yield getLibraryId(libraryId);
        return _updateOrAdd(libraryId, assessment, base);
    });
}
exports.updateOrAdd = updateOrAdd;
function _updateOrAdd(libraryId, assessment, base) {
    return __awaiter(this, void 0, void 0, function* () {
        const search = new Map();
        search.set(assessmentsTypes_1.API_ID_TAG, assessment.getId());
        const assessments = yield find(libraryId, search);
        const isNew = (assessments.length == 0);
        if (isNew) {
            console.log(`new ${assessment.details.name}`);
            yield _publishAssessment(libraryId, assessment, true, base);
        }
        else {
            console.log(`${assessment.details.name} exists`);
            const libraryAssessment = assessments[0];
            const checksumLibrary = libraryAssessment.metadata.tags.get(assessmentsTypes_1.API_HASH_TAG);
            const checksumProject = assessment.getHash();
            if (checksumLibrary != checksumProject) {
                console.log(`${assessment.details.name} updating 
      new "${checksumProject}" old "${checksumLibrary}"`);
                assessment.assessmentId = libraryAssessment.assessmentId;
                yield _publishAssessment(libraryId, assessment, false, base);
            }
            else {
                console.log(`${assessment.details.name} unchanged`);
            }
        }
    });
}
function loadProjectAssessments(dir) {
    return __awaiter(this, void 0, void 0, function* () {
        // read assessments
        const res = [];
        const assessmentJsonFiles = yield (0, glob_promise_1.default)('*.json', { cwd: path_1.default.join(dir, ASSESSMENTS_DIR), nodir: true });
        let assessments = [];
        for (const file of assessmentJsonFiles) {
            const filePath = path_1.default.join(dir, ASSESSMENTS_DIR, file);
            const assessmentString = yield fs_1.default.promises.readFile(filePath, { encoding: 'utf8' });
            const assessment = JSON.parse(assessmentString);
            assessments = assessments.concat(assessment);
        }
        const rootMetadata = tools_1.default.readMetadataFile(path_1.default.join(dir, ROOT_METADATA));
        const guidesStructure = tools_1.default.getGuidesStructure(rootMetadata, dir, '');
        const metadataPages = getMetadataPages(dir, guidesStructure);
        for (const json of assessments) {
            try {
                const item = (0, assessmentsTypes_1.parse)(json, metadataPages);
                item.basePath = dir;
                res.push(item);
            }
            catch (error) {
                console.log(`Skipping assessment ${error.message}`);
            }
        }
        console.log(`Assessments found: ${res.length}`);
        return res;
    });
}
function getMetadataPages(dir, guidesStructure) {
    let metadataPages = [];
    for (const data of guidesStructure) {
        if (data.type === 'page') {
            const filePath = path_1.default.join(dir, data['content_path']);
            const page = fs_1.default.readFileSync(filePath, { encoding: "utf-8" });
            metadataPages.push({ page, data });
        }
        if (data.children) {
            const res = getMetadataPages(dir, data.children);
            metadataPages = metadataPages.concat(res);
        }
    }
    return metadataPages;
}
function fromCodioProject(libraryId, path) {
    return __awaiter(this, void 0, void 0, function* () {
        yield (0, tools_1.fixGuidesVersion)(path);
        libraryId = yield getLibraryId(libraryId);
        const assessments = yield loadProjectAssessments(path);
        for (const _ of assessments) {
            try {
                yield _updateOrAdd(libraryId, _, path);
            }
            catch (error) {
                console.error(error.message);
            }
        }
    });
}
function find(libraryId, tags = new Map()) {
    return __awaiter(this, void 0, void 0, function* () {
        libraryId = yield getLibraryId(libraryId);
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const params = tools_1.default.mapToObject(tags);
            const urlParams = new URLSearchParams(params);
            const url = `${(0, tools_1.getApiV1Url)()}/assessment_library/${libraryId}/assessment?${urlParams.toString()}`;
            const apiRes = yield getJson(url, undefined, (0, tools_1.getBearer)());
            if (!apiRes.assessments) {
                return [];
            }
            const res = [];
            for (const _ of apiRes.assessments) {
                res.push((0, assessmentsTypes_1.parseApi)(_));
            }
            return res;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
const assessment = {
    listLibraries,
    find,
    updateOrAdd,
    fromCodioProject,
    publishAssessment,
};
exports.default = assessment;
