export declare const API_ID_TAG = "CODIO_API_ID";
export declare const API_HASH_TAG = "CODIO_API_HASH";
export declare const BLOOMS_LEVEL: {
    '1': string;
    '2': string;
    '3': string;
    '4': string;
    '5': string;
    '6': string;
};
export declare type MetadataPage = {
    data: any;
    page: string;
};
declare type LayoutOpened = {
    type: string;
    panelNumber: number;
    content: string;
};
export declare class Assessment {
    type: string;
    basePath: string;
    taskId: string;
    assessmentId: string | undefined;
    details: {
        name: string;
        showName: boolean;
        instructions: string;
        showExpectedAnswer: boolean;
        guidance: string;
        maxAttemptsCount: number;
    };
    metadata: {
        opened: LayoutOpened[];
        files: string[];
        tags: Map<string, string>;
        layout: string | undefined;
        content: string | undefined;
        pageTitle: string | undefined;
        buttonText: string | undefined;
    };
    body: any;
    private getTagsFromJson;
    private getTagsFromJsonApi;
    getBundle(base: string): Promise<string | undefined>;
    getId(): string;
    private loadFiles;
    getHash(): string;
    private _export;
    export(checksum?: boolean): string;
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentParsons extends Assessment {
    type: string;
    body: {
        parsonPuzzle: {
            initial: string;
            options: string;
            grader: string;
            showGuidanceAfterResponseOption: {
                type: string;
                passedFrom: number | undefined;
            };
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentAdvanced extends Assessment {
    type: string;
    body: {
        codeTest: {
            command: string;
            arePartialPointsAllowed: boolean;
            timeoutSeconds: number;
            showGuidanceAfterResponseOption: {
                type: string;
                passedFrom: number | undefined;
            };
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentMultipleChoice extends Assessment {
    type: string;
    body: {
        multipleChoice: {
            options: {
                id: string;
                correct: boolean;
                answer: string;
            }[];
            isMultipleResponse: boolean;
            isRandomized: boolean;
            arePartialPointsAllowed: boolean;
            showGuidanceAfterResponseOption: {
                type: string;
                passedFrom: number | undefined;
            };
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentGradeBook extends Assessment {
    type: string;
    body: {
        gradeBook: {
            arePartialPointsAllowed: boolean;
            rubrics: {
                id: string;
                weight: number;
                message: string;
            };
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentFreeText extends Assessment {
    type: string;
    body: {
        freeText: {
            previewType: string;
            arePartialPointsAllowed: boolean;
            rubrics: {
                id: string;
                weight: number;
                message: string;
            };
            showGuidanceAfterResponseOption: {
                type: string;
                passedFrom: number | undefined;
            };
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentFillInTheBlanks extends Assessment {
    type: string;
    body: {
        fillInBlanks: {
            text: string;
            showValues: boolean;
            blanks: string[];
            texts: string[];
            distractors: string;
            arePartialPointsAllowed: boolean;
            showGuidanceAfterResponseOption: {
                type: string;
                passedFrom: number | undefined;
            };
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentFreeTextAuto extends Assessment {
    type: string;
    body: {
        freeTextAuto: {
            previewType: string;
            arePartialPointsAllowed: boolean;
            command: string;
            timeout: number;
            showGuidanceAfterResponseOption: {
                type: string;
                passedFrom: number | undefined;
            };
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare class AssessmentStandardCode extends Assessment {
    type: string;
    body: {
        codeCompare: {
            options: {
                ignoreCase: boolean;
                ignoreWhitespaces: boolean;
                ignoreNewline: boolean;
                matchSubstring: boolean;
                timeout: number;
            };
            command: string;
            preExecuteCommand: string;
            showGuidanceAfterResponseOption: {
                type: string;
                passedFrom: number | undefined;
            };
            sequence: {
                arguments: string;
                input: string;
                output: string;
                showFeedback: boolean;
                feedback: string;
            }[];
        };
    };
    constructor(json: any, metadata?: MetadataPage[]);
}
export declare function parse(json: any, metadataPages: MetadataPage[]): Assessment;
export declare function parseApi(json: any): Assessment;
export {};
