export declare type PathMap = {
    source: string;
    destination: string;
};
export declare type TimeExtension = {
    extendedDeadline?: number | undefined;
    extendedTimeLimit?: number | undefined;
};
export declare type AssignmentSettings = {
    enableResetAssignmentByStudent?: boolean;
    disableDownloadByStudent?: boolean;
    visibilityOnDisabled?: string;
    visibilityOnCompleted?: string;
    startTime?: Date | null;
    endTime?: Date | null;
    action?: string;
    dueTime?: Date | null;
    markAsCompleteOnDueDate?: boolean;
    penaltiesV2?: PenaltySettings;
    examMode?: {
        timedExamMode: {
            enabled: boolean;
            duration: number;
        };
        shuffleQuestionsOrder: boolean;
        forwardOnlyNavigation: boolean;
        singleLogin: boolean;
        authentication: boolean;
    };
    releaseGrades?: boolean;
    isDisabled?: boolean;
};
export declare type PenaltySettings = {
    enable: boolean;
    deductionIntervalMinutes: number;
    deductionPercent: number;
    lowestGradePercent: number;
};
declare type PublishOptions = {
    changelog: string;
    stack: string;
    withStackUpdate: boolean;
};
declare function publishArchive(courseId: string, assignmentId: string, archivePath: string, changelogOrOptions: string | PublishOptions): Promise<void>;
declare function reducePublish(courseId: string, srcDir: string, yamlDir: string, changelogOrOptions: string | PublishOptions): Promise<void>;
export declare function getSettings(courseId: string, assignmentId: string): Promise<AssignmentSettings>;
export declare function updateSettings(courseId: string, assignmentId: string, settings: AssignmentSettings): Promise<AssignmentSettings>;
export declare function updateStudentTimeExtension(courseId: string, assignmentId: string, studentId: string, extension: TimeExtension): Promise<void>;
declare const assignment: {
    publish: (courseId: string, assignmentId: string, projectPath: string, changelogOrOptions: string | PublishOptions) => Promise<void>;
    publishArchive: typeof publishArchive;
    reducePublish: typeof reducePublish;
    updateSettings: typeof updateSettings;
    getSettings: typeof getSettings;
    updateStudentTimeExtension: typeof updateStudentTimeExtension;
};
export default assignment;
