"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateStudentTimeExtension = exports.updateSettings = exports.getSettings = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const bentWrapper_1 = __importDefault(require("./bentWrapper"));
const form_data_1 = __importDefault(require("form-data"));
const tar_1 = __importDefault(require("tar"));
const glob_promise_1 = __importDefault(require("glob-promise"));
const yaml_1 = __importDefault(require("yaml"));
const tools_1 = __importStar(require("./tools"));
const config_1 = __importStar(require("./config"));
const lodash_1 = __importDefault(require("lodash"));
const course_1 = require("./course");
function archiveTar(src) {
    return __awaiter(this, void 0, void 0, function* () {
        const dir = yield fs_1.default.promises.mkdtemp('/tmp/codio_export');
        const file = path_1.default.join(dir, 'project.tar.gz');
        yield tar_1.default.c({
            gzip: true,
            file,
            cwd: src,
            filter: (path) => {
                for (const exclude of config_1.excludePaths) {
                    if (lodash_1.default.startsWith(path, exclude)) {
                        return false;
                    }
                }
                return true;
            }
        }, ['./']);
        return { file, dir };
    });
}
const sleep = (seconds) => __awaiter(void 0, void 0, void 0, function* () { return new Promise((resolve) => setTimeout(resolve, seconds * 1000)); });
const getJson = (0, bentWrapper_1.default)();
function publishArchive(courseId, assignmentId, archivePath, changelogOrOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST');
            const postData = new form_data_1.default();
            if (typeof changelogOrOptions == 'string') {
                postData.append('changelog', changelogOrOptions || '');
            }
            else {
                const options = changelogOrOptions || {};
                postData.append('changelog', options.changelog || '');
                postData.append('stackVersionId', options.stack || '');
                postData.append('withStackUpdate', `${options.withStackUpdate}`);
            }
            postData.append('archive', fs_1.default.createReadStream(archivePath), {
                knownLength: fs_1.default.statSync(archivePath).size
            });
            const headers = Object.assign(postData.getHeaders(), (0, tools_1.getBearer)());
            headers['Content-Length'] = postData.getLengthSync();
            const res = yield api(`/courses/${courseId}/assignments/${assignmentId}/versions`, postData, headers);
            const taskUrl = res['taskUri'];
            if (!taskUrl) {
                throw new Error('task Url not found');
            }
            for (let i = 0; i < 100; i++) { // 100 checks attempt
                yield sleep(10);
                const status = yield getJson(taskUrl, undefined, (0, tools_1.getBearer)());
                console.log(status);
                if (status['done']) {
                    if (status['error']) {
                        throw new Error(status['error']);
                    }
                    break;
                }
            }
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
function validityState(ymls) {
    const map = new Map();
    for (const yml of ymls) {
        if (!yml.section) {
            console.error(`Warning: ${yml.assignment || yml.assignmentName} is empty`);
            yml.section = [];
        }
        const section = lodash_1.default.isString(yml.section) ? [yml.section] : yml.section;
        const assignmentId = yml.assignment || yml.assignmentName;
        if (assignmentId === undefined) {
            throw new Error('assignment and assignmentName does not exist');
        }
        if (map.has(assignmentId)) {
            const item = map.get(assignmentId);
            if (!item) {
                continue;
            }
            item.section.push(section);
            item.paths = item.paths.concat(yml.paths || []);
        }
        else {
            map.set(assignmentId, {
                assignment: yml.assignment,
                assignmentName: yml.assignmentName,
                paths: yml.paths || [],
                section: [section]
            });
        }
    }
    return Array.from(map.values());
}
// after find names should regroup assignments by id
function validateYmlCfg(ymls) {
    const map = new Map();
    for (const yml of ymls) {
        const section = yml.section || [];
        const assignmentId = yml.assignment;
        if (assignmentId === undefined) {
            throw new Error('assignment does not exist');
        }
        if (map.has(assignmentId)) {
            const item = map.get(assignmentId);
            if (!item) {
                continue;
            }
            item.section = item.section.concat(section);
            item.paths = item.paths.concat(yml.paths || []);
        }
        else {
            map.set(assignmentId, {
                assignment: yml.assignment,
                assignmentName: yml.assignmentName,
                paths: yml.paths || [],
                section: section
            });
        }
    }
    return Array.from(map.values());
}
function loadYaml(yamlDir) {
    return __awaiter(this, void 0, void 0, function* () {
        let res = [];
        const files = yield (0, glob_promise_1.default)('*.+(yml|yaml)', { cwd: yamlDir, nodir: true });
        for (const file of files) {
            const ymlText = yield fs_1.default.promises.readFile(path_1.default.join(yamlDir, file), { encoding: 'utf-8' });
            let ymls = yaml_1.default.parse(ymlText);
            if (!ymls) {
                continue;
            }
            if (!lodash_1.default.isArray(ymls)) {
                ymls = [ymls];
            }
            res = res.concat(ymls);
        }
        return validityState(res);
    });
}
function findNames(courseId, ymlCfg) {
    return __awaiter(this, void 0, void 0, function* () {
        const usesNames = (0, lodash_1.default)(ymlCfg).map('assignmentName').compact().size() > 0;
        if (!usesNames) {
            return;
        }
        const course = yield (0, course_1.info)(courseId);
        for (const item of ymlCfg) {
            if (!item.assignment && item.assignmentName) { // make id higher priority
                const assignments = lodash_1.default.filter(course.assignments, { name: item.assignmentName });
                if (assignments.length == 0) {
                    throw new Error(`no assignments in course with name ${item.assignmentName} is found`);
                }
                if (assignments.length > 1) {
                    throw new Error(`many assignments in course with same name ${item.assignmentName}`);
                }
                item.assignment = assignments[0].id;
            }
        }
    });
}
function reducePublish(courseId, srcDir, yamlDir, changelogOrOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        let ymlCfg = yield loadYaml(yamlDir);
        yield findNames(courseId, ymlCfg);
        ymlCfg = validateYmlCfg(ymlCfg);
        for (const item of ymlCfg) {
            console.log(`publishing ${JSON.stringify(item)}`);
            const tmpDstDir = fs_1.default.mkdtempSync('/tmp/publish_codio_reduce');
            const paths = item.paths || [];
            paths.push(`!${yamlDir}`); // exclude yaml directory from export
            paths.push(`!${yamlDir}/**`); // exclude yaml directory from export
            if (!item.assignment) {
                throw new Error(`assignment not found with name "${item.assignmentName}"`);
            }
            yield tools_1.default.reduce(srcDir, tmpDstDir, item.section, lodash_1.default.compact(paths));
            yield assignment.publish(courseId, item.assignment, tmpDstDir, changelogOrOptions);
            fs_1.default.rmSync(tmpDstDir, { recursive: true });
        }
    });
}
function fromRawSettings(res) {
    var _a, _b, _c, _d, _e;
    return {
        enableResetAssignmentByStudent: res.enableResetAssignmentByStudent,
        disableDownloadByStudent: res.disableDownloadByStudent,
        visibilityOnDisabled: res.visibilityOnDisabled,
        visibilityOnCompleted: res.visibilityOnCompleted,
        startTime: convertDateToLocal(res.startTime),
        endTime: convertDateToLocal(res.endTime),
        action: res.action,
        dueTime: convertDateToLocal(res.dueTime),
        markAsCompleteOnDueDate: res.markAsCompleteOnDueDate,
        penaltiesV2: res.penaltiesV2,
        examMode: res.examMode ? {
            timedExamMode: (_a = res.examMode) === null || _a === void 0 ? void 0 : _a.timedMode,
            shuffleQuestionsOrder: (_b = res.examMode) === null || _b === void 0 ? void 0 : _b.shuffleQuestionsOrder,
            forwardOnlyNavigation: (_c = res.examMode) === null || _c === void 0 ? void 0 : _c.forwardOnlyNavigation,
            singleLogin: (_d = res.examMode) === null || _d === void 0 ? void 0 : _d.singleLogin,
            authentication: (_e = res.examMode) === null || _e === void 0 ? void 0 : _e.authentication
        } : undefined,
        releaseGrades: res.releaseGrades,
        isDisabled: res.isDisabled,
    };
}
function getSettings(courseId, assignmentId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)());
            const rawSettings = yield api(`/courses/${courseId}/assignments/${assignmentId}/settings`, undefined, (0, tools_1.getBearer)());
            return fromRawSettings(rawSettings);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.getSettings = getSettings;
function convertDateToLocal(date) {
    if (!date) {
        return null;
    }
    return new Date(date);
}
function toRawSettings(settings) {
    const res = {};
    if (settings.enableResetAssignmentByStudent !== undefined) {
        res.enableResetAssignmentByStudent = settings.enableResetAssignmentByStudent;
    }
    if (settings.disableDownloadByStudent !== undefined) {
        res.disableDownloadByStudent = settings.disableDownloadByStudent;
    }
    if (settings.visibilityOnDisabled !== undefined) {
        res.visibilityOnDisabled = settings.visibilityOnDisabled;
    }
    if (settings.visibilityOnCompleted !== undefined) {
        res.visibilityOnCompleted = settings.visibilityOnCompleted;
    }
    if (settings.startTime !== undefined) {
        res.startTime = settings.startTime ? settings.startTime.toISOString() : '';
    }
    if (settings.endTime !== undefined) {
        res.endTime = settings.endTime ? settings.endTime.toISOString() : '';
    }
    if (settings.action !== undefined) {
        res.action = settings.action;
    }
    if (settings.dueTime !== undefined) {
        res.dueTime = settings.dueTime ? settings.dueTime.toISOString() : '';
    }
    if (settings.markAsCompleteOnDueDate !== undefined) {
        res.markAsCompleteOnDueDate = settings.markAsCompleteOnDueDate;
    }
    if (settings.penaltiesV2 !== undefined) {
        const deductionPercent = settings.penaltiesV2.deductionPercent;
        if (!lodash_1.default.isNumber(deductionPercent) || deductionPercent < 0 || deductionPercent > 100) {
            throw new Error("penalty deduction percent must be a number between 0 and 100");
        }
        const lowestGradePercent = settings.penaltiesV2.lowestGradePercent;
        if (!lodash_1.default.isNumber(lowestGradePercent) || lowestGradePercent < 0 || lowestGradePercent > 100) {
            throw new Error("lowest grade percent after penalty must be a number between 0 and 100");
        }
        res.penaltiesV2 = settings.penaltiesV2;
    }
    if (settings.examMode !== undefined) {
        res.examMode = {
            timedMode: settings.examMode.timedExamMode,
            shuffleQuestionsOrder: settings.examMode.shuffleQuestionsOrder,
            forwardOnlyNavigation: settings.examMode.forwardOnlyNavigation,
            singleLogin: settings.examMode.singleLogin,
            authentication: settings.examMode.authentication
        };
    }
    if (settings.releaseGrades !== undefined) {
        res.releaseGrades = settings.releaseGrades;
    }
    if (settings.isDisabled !== undefined) {
        res.isDisabled = settings.isDisabled;
    }
    return res;
}
function updateSettings(courseId, assignmentId, settings) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST');
            const res = yield api(`/courses/${courseId}/assignments/${assignmentId}/settings`, toRawSettings(settings), (0, tools_1.getBearer)());
            return fromRawSettings(res);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.updateSettings = updateSettings;
function updateStudentTimeExtension(courseId, assignmentId, studentId, extension) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST');
            return yield api(`/courses/${courseId}/assignments/${assignmentId}/students/${studentId}`, extension, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.updateStudentTimeExtension = updateStudentTimeExtension;
const assignment = {
    publish: (courseId, assignmentId, projectPath, changelogOrOptions) => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, tools_1.fixGuidesVersion)(projectPath);
        const { file, dir } = yield archiveTar(projectPath);
        yield assignment.publishArchive(courseId, assignmentId, file, changelogOrOptions);
        fs_1.default.rmSync(dir, { recursive: true });
    }),
    publishArchive,
    reducePublish,
    updateSettings,
    getSettings,
    updateStudentTimeExtension
};
exports.default = assignment;
