"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bent_1 = __importDefault(require("bent"));
const RETRIES_COUNT = 5;
const waitTimeout = (ms) => __awaiter(void 0, void 0, void 0, function* () {
    yield new Promise((resolve) => {
        setTimeout(resolve, ms);
    });
});
const encodings = new Set(['json', 'buffer', 'string']);
const getEncodingFromArgs = (...args) => {
    let encoding;
    args.forEach(arg => {
        if (typeof arg === 'string') {
            if (arg.toUpperCase() === arg) {
                // ignore
            }
            else if (arg.startsWith('http:') || arg.startsWith('https:')) {
                // ignore
            }
            else {
                if (encodings.has(arg)) {
                    encoding = arg;
                }
            }
        }
    });
    if (!encoding) {
        encoding = 'json';
    }
    return encoding;
};
exports.default = (...args) => {
    const api = (0, bent_1.default)(...args);
    const req = (url, body, headers, tries) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            if (tries === 0) {
                throw Error('Tries limit exceeded');
            }
            const resp = yield api(url, body, headers);
            const encoding = getEncodingFromArgs(args);
            if (!encoding) {
                return resp;
            }
            if (encoding === 'buffer' && resp.arrayBuffer) {
                return resp.arrayBuffer();
            }
            if (encoding === 'json' && resp.json) {
                return resp.json();
            }
            if (encoding === 'string' && resp.text) {
                return resp.text();
            }
            return resp.json ? resp.json() : resp;
        }
        catch (e) {
            if (e.statusCode === 429) {
                const dailyRemaining = e.headers['x-ratelimit-dailylimit-remaining'];
                if (parseInt(dailyRemaining) === 0) {
                    throw e;
                }
                const resetHeaderMs = e.headers['x-ratelimit-reset'] / 1000000;
                yield waitTimeout(resetHeaderMs);
                const triesCount = tries !== undefined && tries > 0 ? tries - 1 : RETRIES_COUNT;
                return req(url, body, headers, triesCount);
            }
            throw e;
        }
    });
    return req;
};
