export declare type Assignment = {
    id: string;
    name: string;
};
export declare type Module = {
    id: string;
    name: string;
    assignments: Assignment[];
};
export declare type Course = {
    id: string;
    name: string;
    modules: Module[];
    assignments: Assignment[];
    creationDate: Date;
    archivedDate: Date;
    archived: boolean;
};
export declare type StudentProgress = {
    student_id: string;
    student_email: string;
    student_name: string;
    seconds_spent: number;
    grade: number;
    extra_credit: number;
    status: 'NOT_STARTED' | 'COMPLETED' | 'STARTED';
    completion_date: Date;
    timeLimitExtension: number | undefined;
    deadlineExtension: number | undefined;
};
export declare type AssignmentProgressInfo = {
    seconds_spent: number;
    grade: number;
    status: 'COMPLETED' | 'STARTED';
    completion_date: Date;
    extended_deadline: number | undefined;
    extended_time_limit: number | undefined;
};
export declare type AssignmentProgress = {
    assignment_name: string;
    assignment_id: string;
    module_name: string;
    module_id: string;
    started: boolean;
    progress: AssignmentProgressInfo | undefined;
};
export declare type User = {
    id: string;
    name: string;
    login: string;
    email: string;
};
export declare type CourseExport = {
    taskId: string;
    done: boolean;
    error?: string;
    url?: string;
};
export declare type TaskResponce = {
    taskId: string;
};
export declare function info(courseId: string, withHiddenAssignments?: boolean): Promise<Course>;
export declare function findByName(courseName: string, withHiddenAssignments: boolean | undefined): Promise<Course>;
export declare function assignmentStudentsProgress(courseId: string, assignmentId: string): Promise<StudentProgress[]>;
export declare function studentCourseProgress(courseId: string, studentIdentification: string): Promise<AssignmentProgress[]>;
export declare function waitDownloadTask(taskUrl: string): Promise<string>;
export declare function exportStudentAssignment(courseId: string, assignmentId: string, studentId: string): Promise<string>;
export declare function downloadStudentAssignment(courseId: string, assignmentId: string, studentId: string, filePath: string): Promise<void>;
export declare function downloadStudentCSV(courseId: string, studentId: string, filePath: string): Promise<void>;
export declare function downloadAssignmentCSV(courseId: string, assignmentId: string, filePath: string): Promise<void>;
export declare function downloadAssessmentData(courseId: string, assignmentIds: string, filePath: string): Promise<void>;
export declare function exportStudentCSV(courseId: string, studentId: string): Promise<string>;
export declare function exportAssignmentCSV(courseId: string, assignmentId: string): Promise<string>;
export declare function exportAssessmentData(courseId: string, assignmentIds: string): Promise<string>;
export declare function getStudents(courseId: string): Promise<User[]>;
export declare function getTeachers(courseId: string): Promise<User[]>;
export declare function getSourceExports(courseId: string): Promise<CourseExport[]>;
export declare function getSourceExportProgress(courseId: string, taskId: string): Promise<CourseExport>;
export declare function createSourceExport(courseId: string): Promise<CourseExport>;
export declare function downloadSourceExport(courseId: string, filePath: string): Promise<void>;
export declare function getWorkExports(courseId: string): Promise<CourseExport[]>;
export declare function getWorkExportProgress(courseId: string, taskId: string): Promise<CourseExport>;
export declare function createWorkExport(courseId: string): Promise<CourseExport>;
export declare function downloadWorkExport(courseId: string, filePath: string): Promise<void>;
export declare type ListCoursesResponse = {
    courses: Course[];
    nextToken: string;
};
export declare function list(nextToken: string, archived?: boolean): Promise<ListCoursesResponse>;
export declare type ArchiveResponse = {
    archivedDate: string;
};
export declare function archive(courseId: string): Promise<Date>;
export declare function exportCoachData(courseId: string): Promise<string>;
export declare function exportLLMProxyData(courseId: string): Promise<string>;
declare const course: {
    assignmentStudentsProgress: typeof assignmentStudentsProgress;
    info: typeof info;
    exportStudentAssignment: typeof exportStudentAssignment;
    downloadStudentAssignment: typeof downloadStudentAssignment;
    exportStudentCSV: typeof exportStudentCSV;
    exportAssignmentCSV: typeof exportAssignmentCSV;
    exportAssessmentData: typeof exportAssessmentData;
    downloadStudentCSV: typeof downloadStudentCSV;
    downloadAssignmentCSV: typeof downloadAssignmentCSV;
    downloadAssessmentData: typeof downloadAssessmentData;
    findByName: typeof findByName;
    getStudents: typeof getStudents;
    getTeachers: typeof getTeachers;
    getWorkExportProgress: typeof getWorkExportProgress;
    getWorkExports: typeof getWorkExports;
    createWorkExport: typeof createWorkExport;
    downloadWorkExport: typeof downloadWorkExport;
    getSourceExportProgress: typeof getSourceExportProgress;
    getSourceExports: typeof getSourceExports;
    createSourceExport: typeof createSourceExport;
    downloadSourceExport: typeof downloadSourceExport;
    studentCourseProgress: typeof studentCourseProgress;
    list: typeof list;
    archive: typeof archive;
    exportCoachData: typeof exportCoachData;
    exportLLMProxyData: typeof exportLLMProxyData;
};
export default course;
