"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportLLMProxyData = exports.exportCoachData = exports.archive = exports.list = exports.downloadWorkExport = exports.createWorkExport = exports.getWorkExportProgress = exports.getWorkExports = exports.downloadSourceExport = exports.createSourceExport = exports.getSourceExportProgress = exports.getSourceExports = exports.getTeachers = exports.getStudents = exports.exportAssessmentData = exports.exportAssignmentCSV = exports.exportStudentCSV = exports.downloadAssessmentData = exports.downloadAssignmentCSV = exports.downloadStudentCSV = exports.downloadStudentAssignment = exports.exportStudentAssignment = exports.waitDownloadTask = exports.studentCourseProgress = exports.assignmentStudentsProgress = exports.findByName = exports.info = void 0;
const bentWrapper_1 = __importDefault(require("./bentWrapper"));
const https_1 = __importDefault(require("https"));
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const config_1 = __importDefault(require("./config"));
const tools_1 = require("./tools");
const getJson = (0, bentWrapper_1.default)();
function flattenAssignments(course) {
    course.assignments = lodash_1.default.flatten(lodash_1.default.map(course.modules, 'assignments'));
    if (course.creationDate) {
        course.creationDate = new Date(course.creationDate);
    }
    if (course.archivedDate) {
        course.archivedDate = new Date(course.archivedDate);
    }
}
function info(courseId, withHiddenAssignments = true) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const params = {
                withHiddenAssignments: withHiddenAssignments ? 'true' : 'false'
            };
            const urlParams = new URLSearchParams(params);
            const course = yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}?${urlParams.toString()}`, undefined, (0, tools_1.getBearer)());
            flattenAssignments(course);
            return course;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.info = info;
function findByName(courseName, withHiddenAssignments) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const params = {
                name: courseName,
                withHiddenAssignments: withHiddenAssignments ? 'true' : 'false'
            };
            const urlParams = new URLSearchParams(params);
            const course = yield getJson(`${(0, tools_1.getApiV1Url)()}/courses?${urlParams.toString()}`, undefined, (0, tools_1.getBearer)());
            flattenAssignments(course);
            return course;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.findByName = findByName;
function assignmentStudentsProgress(courseId, assignmentId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const res = yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/assignments/${assignmentId}/students`, undefined, (0, tools_1.getBearer)());
            for (const progress of res) {
                if (progress.completion_date) {
                    progress.completion_date = (0, tools_1.secondsToDate)(progress.completion_date.seconds);
                }
            }
            return res;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.assignmentStudentsProgress = assignmentStudentsProgress;
function studentCourseProgress(courseId, studentIdentification) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const res = yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/students/${studentIdentification}/progress`, undefined, (0, tools_1.getBearer)());
            for (const progress of res) {
                if (progress.completion_date) {
                    progress.completion_date = new Date(progress.completion_date);
                }
            }
            return res;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.studentCourseProgress = studentCourseProgress;
function waitDownloadTask(taskUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const archive = yield getJson(taskUrl, undefined, (0, tools_1.getBearer)());
            if (!archive.done) {
                yield new Promise(resolve => setTimeout(resolve, 500));
                return yield waitDownloadTask(taskUrl);
            }
            if (archive.error) {
                throw new Error(`Task ${archive.taskId} failed with an error`);
            }
            return archive.url;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.waitDownloadTask = waitDownloadTask;
function exportStudentAssignment(courseId, assignmentId, studentId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const res = yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/assignments/${assignmentId}/students/${studentId}/download`, undefined, (0, tools_1.getBearer)());
            const taskUrl = res['taskUri'];
            if (!taskUrl) {
                throw new Error('task Url not found');
            }
            return yield waitDownloadTask(taskUrl);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.exportStudentAssignment = exportStudentAssignment;
function downloadStudentAssignment(courseId, assignmentId, studentId, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const url = yield exportStudentAssignment(courseId, assignmentId, studentId);
        return download(filePath, url);
    });
}
exports.downloadStudentAssignment = downloadStudentAssignment;
function downloadStudentCSV(courseId, studentId, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const url = yield exportStudentCSV(courseId, studentId);
        return download(filePath, url);
    });
}
exports.downloadStudentCSV = downloadStudentCSV;
function downloadAssignmentCSV(courseId, assignmentId, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const url = yield exportAssignmentCSV(courseId, assignmentId);
        return download(filePath, url);
    });
}
exports.downloadAssignmentCSV = downloadAssignmentCSV;
function downloadAssessmentData(courseId, assignmentIds, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const url = yield exportAssessmentData(courseId, assignmentIds);
        return download(filePath, url);
    });
}
exports.downloadAssessmentData = downloadAssessmentData;
function download(filePath, url) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!url) {
            throw new Error('Url Not Found');
        }
        const file = fs_1.default.createWriteStream(filePath);
        return new Promise((resolve, reject) => {
            https_1.default.get(url, (response) => {
                response.pipe(file);
                file.on('finish', () => {
                    file.close();
                    resolve();
                });
            }).on('error', (err) => {
                fs_1.default.unlink(filePath, e => {
                    reject(e && e.message);
                });
                reject(err.message);
            });
        });
    });
}
function exportStudentCSV(courseId, studentId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/students/${studentId}/export/csv`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.exportStudentCSV = exportStudentCSV;
function exportAssignmentCSV(courseId, assignmentId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/assignments/${assignmentId}/export/csv`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.exportAssignmentCSV = exportAssignmentCSV;
function exportAssessmentData(courseId, assignmentIds) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const res = yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/export/assessments/csv?assignmentIds=${assignmentIds}`, undefined, (0, tools_1.getBearer)());
            const taskUrl = res['taskUri'];
            if (!taskUrl) {
                throw new Error('task Url not found');
            }
            return yield waitDownloadTask(taskUrl);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.exportAssessmentData = exportAssessmentData;
function getStudents(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/students`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.getStudents = getStudents;
function getTeachers(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/teachers`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.getTeachers = getTeachers;
function getSourceExports(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/export/sources`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.getSourceExports = getSourceExports;
function getSourceExportProgress(courseId, taskId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/export/sources/progress/${taskId}`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.getSourceExportProgress = getSourceExportProgress;
function waitExportTaskReady(taskId, taskChecker) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            const checker = () => __awaiter(this, void 0, void 0, function* () {
                try {
                    const check = yield taskChecker(taskId);
                    if (check.done) {
                        if (check.error) {
                            reject(new Error(check.error));
                        }
                        else {
                            resolve(check);
                        }
                    }
                    else {
                        setTimeout(() => checker(), 10000);
                    }
                }
                catch (ex) {
                    reject(ex);
                }
            });
            checker();
        });
    });
}
function createSourceExport(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST', 'json', 200);
        try {
            const res = yield api(`/courses/${courseId}/export/sources`, undefined, (0, tools_1.getBearer)());
            return yield waitExportTaskReady(res.taskId, (taskId) => getSourceExportProgress(courseId, taskId));
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.createSourceExport = createSourceExport;
function downloadSourceExport(courseId, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const resp = yield createSourceExport(courseId);
        if (!resp.url) {
            throw new Error(resp.error);
        }
        return download(filePath, resp.url);
    });
}
exports.downloadSourceExport = downloadSourceExport;
function getWorkExports(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/export/workdata`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.getWorkExports = getWorkExports;
function getWorkExportProgress(courseId, taskId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield getJson(`${(0, tools_1.getApiV1Url)()}/courses/${courseId}/export/workdata/progress/${taskId}`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.getWorkExportProgress = getWorkExportProgress;
function createWorkExport(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST', 'json', 200);
        try {
            const res = yield api(`/courses/${courseId}/export/workdata`, undefined, (0, tools_1.getBearer)());
            return yield waitExportTaskReady(res.taskId, (taskId) => getWorkExportProgress(courseId, taskId));
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.createWorkExport = createWorkExport;
function downloadWorkExport(courseId, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const resp = yield createWorkExport(courseId);
        if (!resp.url) {
            throw new Error(resp.error);
        }
        return download(filePath, resp.url);
    });
}
exports.downloadWorkExport = downloadWorkExport;
function list(nextToken, archived) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const params = {};
            if (nextToken) {
                params.nextToken = nextToken;
            }
            if (archived !== undefined) {
                params.archived = archived ? 'true' : 'false';
            }
            const urlParams = new URLSearchParams(params);
            const resp = yield getJson(`${(0, tools_1.getApiV1Url)()}/courses_list?${urlParams.toString()}`, undefined, (0, tools_1.getBearer)());
            lodash_1.default.forEach(resp.courses, course => flattenAssignments(course));
            return resp;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.list = list;
function archive(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST', 'json', 200);
            const resp = yield api(`/courses/${courseId}/archive`, undefined, (0, tools_1.getBearer)());
            return new Date(resp.archivedDate);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.archive = archive;
function exportCoachData(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST', 'json', 200);
            const res = yield api(`/courses/${courseId}/export/coach`, undefined, (0, tools_1.getBearer)());
            const taskUrl = res['taskUri'];
            if (!taskUrl) {
                throw new Error('task Url not found');
            }
            return yield waitDownloadTask(taskUrl);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.exportCoachData = exportCoachData;
function exportLLMProxyData(courseId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST', 'json', 200);
            const res = yield api(`/courses/${courseId}/export/llmproxy`, undefined, (0, tools_1.getBearer)());
            const taskUrl = res['taskUri'];
            if (!taskUrl) {
                throw new Error('task Url not found');
            }
            return yield waitDownloadTask(taskUrl);
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.exportLLMProxyData = exportLLMProxyData;
const course = {
    assignmentStudentsProgress,
    info,
    exportStudentAssignment,
    downloadStudentAssignment,
    exportStudentCSV,
    exportAssignmentCSV,
    exportAssessmentData,
    downloadStudentCSV,
    downloadAssignmentCSV,
    downloadAssessmentData,
    findByName,
    getStudents,
    getTeachers,
    getWorkExportProgress,
    getWorkExports,
    createWorkExport,
    downloadWorkExport,
    getSourceExportProgress,
    getSourceExports,
    createSourceExport,
    downloadSourceExport,
    studentCourseProgress,
    list,
    archive,
    exportCoachData,
    exportLLMProxyData
};
exports.default = course;
