"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadAllEvents = exports.loadEvents = void 0;
const querystring_1 = __importDefault(require("querystring"));
const bentWrapper_1 = __importDefault(require("./bentWrapper"));
const config_1 = __importDefault(require("./config"));
const tools_1 = require("./tools");
const getJson = (0, bentWrapper_1.default)();
function loadEvents(nextToken, limit = 50) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        const query = {};
        if (limit && (limit >= 10 && limit <= 200)) {
            query.limit = limit;
        }
        if (nextToken) {
            query.nextToken = nextToken;
        }
        const paramsString = querystring_1.default.encode(query);
        const url = `${(0, tools_1.getApiV1Url)()}/events?${paramsString}`;
        try {
            return getJson(url, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.loadEvents = loadEvents;
function loadAllEvents() {
    return __awaiter(this, void 0, void 0, function* () {
        let events = [];
        let nextToken = '';
        do {
            const resp = yield loadEvents(nextToken);
            events = events.concat(resp.events);
            nextToken = resp.nextToken;
        } while (nextToken);
        return events;
    });
}
exports.loadAllEvents = loadAllEvents;
const events = {
    loadAllEvents,
    loadEvents
};
exports.default = events;
