"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.waitTask = exports.publish = exports.info = void 0;
const bentWrapper_1 = __importDefault(require("./bentWrapper"));
const form_data_1 = __importDefault(require("form-data"));
const fs_1 = __importDefault(require("fs"));
const config_1 = __importDefault(require("./config"));
const tools_1 = require("./tools");
const getJson = (0, bentWrapper_1.default)();
function info(stackId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            return getJson(`${(0, tools_1.getApiV1Url)()}/stacks/${stackId}`, undefined, (0, tools_1.getBearer)());
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.info = info;
function publish(stackId, id, provisioner, content, archivePath, message) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const api = (0, bentWrapper_1.default)((0, tools_1.getApiV1Url)(), 'POST', 200);
            const postData = new form_data_1.default();
            postData.append('provisioner', provisioner);
            if (archivePath !== null) {
                postData.append('files', fs_1.default.createReadStream(archivePath), {
                    knownLength: fs_1.default.statSync(archivePath).size
                });
            }
            if (id !== null) {
                postData.append('id', id);
            }
            if (content !== null) {
                postData.append('content', content);
            }
            postData.append('message', message);
            const headers = Object.assign(postData.getHeaders(), (0, tools_1.getBearer)());
            headers['Content-Length'] = yield postData.getLengthSync();
            const res = yield api(`/stacks/${stackId}/versions`, postData, headers);
            return res;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.publish = publish;
function waitTask(taskUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!config_1.default) {
            throw new Error('No Config');
        }
        try {
            const res = yield getJson(taskUrl, undefined, (0, tools_1.getBearer)());
            console.log('waitTask', res);
            if (res.status !== 'done' && res.status !== 'error') {
                yield new Promise(resolve => setTimeout(resolve, 5000));
                return yield waitTask(taskUrl);
            }
            return res;
        }
        catch (error) {
            if (error.json) {
                const message = JSON.stringify(yield error.json());
                throw new Error(message);
            }
            throw error;
        }
    });
}
exports.waitTask = waitTask;
const stack = {
    info,
    publish,
    waitTask
};
exports.default = stack;
