"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBearer = exports.getApiV1Url = exports.convertToGuidesV3 = exports.secondsToDate = exports.createTar = exports.mapToObject = exports.readMetadataFile = exports.getGuidesStructure = exports.reduce = exports.fixGuidesVersion = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const recursive_copy_1 = __importDefault(require("recursive-copy"));
const child_process_1 = __importDefault(require("child_process"));
const config_1 = require("./config");
const tar_1 = __importDefault(require("tar"));
const simple_zstd_1 = require("simple-zstd");
const config_2 = __importDefault(require("./config"));
const CONVERTER_VERSION = '4ca4944ddf9d4fe4df9697bec06cbd0a6c170419';
const GUIDES_CONTENT_DIR = '.guides/content';
const OLD_METADATA_FILE = '.guides/metadata.json';
const INDEX_METADATA_FILE = 'index.json';
const PAGE = 'page';
function fixGuidesVersion(projectPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (fs_1.default.existsSync(path_1.default.join(projectPath, OLD_METADATA_FILE))) {
            yield convertToGuidesV3(projectPath);
        }
    });
}
exports.fixGuidesVersion = fixGuidesVersion;
function reduce(srcDir, dstDir, yaml_sections, paths) {
    return __awaiter(this, void 0, void 0, function* () {
        yield fixGuidesVersion(srcDir);
        const contentDir = path_1.default.join(srcDir, GUIDES_CONTENT_DIR);
        const rootMetadataPath = path_1.default.join(contentDir, INDEX_METADATA_FILE);
        const rootMetadata = readMetadataFile(rootMetadataPath);
        const guidesStructure = getGuidesStructure(rootMetadata, srcDir, '');
        const filter = collectFilter(guidesStructure, lodash_1.default.cloneDeep(yaml_sections));
        const strippedStructure = stripStructure(guidesStructure, filter);
        const strippedSectionsIds = getStrippedSectionIds(strippedStructure);
        const excludePaths = getExcludedPaths(guidesStructure, strippedSectionsIds);
        yield copyStripped(srcDir, dstDir, paths.concat(excludePaths));
        yield updateRootMetadata(strippedStructure, rootMetadata, dstDir);
        yield updateMetadata(strippedStructure, dstDir);
    });
}
exports.reduce = reduce;
function getGuidesStructure(metadata, srcDir, currentPath) {
    return lodash_1.default.map(metadata['order'], item => {
        const contentDirPath = path_1.default.join(srcDir, GUIDES_CONTENT_DIR);
        const sectionPath = path_1.default.join(currentPath, item);
        const indexFilePath = path_1.default.join(contentDirPath, sectionPath, INDEX_METADATA_FILE);
        if (!fs_1.default.existsSync(indexFilePath)) {
            const metadataFilePath = path_1.default.join(contentDirPath, `${sectionPath}.json`);
            const sectionMetadata = readMetadataFile(metadataFilePath);
            sectionMetadata['name'] = item;
            sectionMetadata['content_path'] = path_1.default.join(GUIDES_CONTENT_DIR, `${sectionPath}.md`);
            sectionMetadata['metadata_path'] = path_1.default.join(GUIDES_CONTENT_DIR, `${sectionPath}.json`);
            return sectionMetadata;
        }
        else {
            const newMetadataPath = path_1.default.join(currentPath, item);
            const sectionMetadata = readMetadataFile(indexFilePath);
            sectionMetadata['name'] = item;
            sectionMetadata['metadata_path'] = path_1.default.join(GUIDES_CONTENT_DIR, newMetadataPath);
            sectionMetadata['section_path'] = sectionPath;
            sectionMetadata['children'] = getGuidesStructure(sectionMetadata, srcDir, sectionPath);
            return sectionMetadata;
        }
    });
}
exports.getGuidesStructure = getGuidesStructure;
function readMetadataFile(path) {
    try {
        const metadataJson = fs_1.default.readFileSync(path, { encoding: "utf-8" });
        return JSON.parse(metadataJson);
    }
    catch (error) {
        throw new Error(error);
    }
}
exports.readMetadataFile = readMetadataFile;
const DEFAULT_ALL_SECTION = {
    all: true,
    children: {}
};
function collectFilter(guidesStructure, yaml_sections) {
    const filterMap = {
        all: false,
        children: {}
    };
    for (const sectionPath of yaml_sections) {
        if (sectionPath.length === 0) {
            continue;
        }
        const section = traverseItems(guidesStructure, sectionPath, filterMap);
        if (!section) {
            throw new Error(`${section} not found`);
        }
    }
    if (lodash_1.default.isEmpty(lodash_1.default.keys(filterMap))) {
        throw new Error(`Nothing to publish`);
    }
    return filterMap;
}
function traverseItems(structure, sectionPath, filterMap) {
    const sectionName = sectionPath.shift();
    if (!sectionName) {
        return;
    }
    const section = findSection(structure, sectionName);
    if (!section) {
        throw new Error(`section "${sectionName}" is not found`);
    }
    if (filterMap.children[section.id] === undefined) {
        filterMap.children[section.id] = {
            all: false,
            children: {}
        };
    }
    if (sectionPath.length > 0) {
        // fill-in filterMap
        traverseItems(section.children, sectionPath, filterMap.children[section.id]);
    }
    else {
        filterMap.children[section.id].all = true;
    }
    return section;
}
// if filter is empty then add all sections
function stripStructure(guidesStructure, filterMap) {
    const structure = lodash_1.default.cloneDeep(guidesStructure);
    return lodash_1.default.filter(structure, section => {
        if (filterMap.all) {
            return true;
        }
        return lodash_1.default.keys(filterMap.children).includes(section.id);
    }).map(section => {
        if (section.children) {
            const filter = filterMap.all ? DEFAULT_ALL_SECTION : filterMap.children[section.id];
            section.children = stripStructure(section.children, filter);
        }
        return section;
    });
}
function findSection(structure, title) {
    const capitalTitle = lodash_1.default.upperCase(title);
    if (structure.title === PAGE) {
        if (structure.title === capitalTitle) {
            return structure;
        }
    }
    else {
        for (const item of structure) {
            if (lodash_1.default.upperCase(item.title) === capitalTitle) {
                return item;
            }
        }
    }
    return undefined;
}
function getStrippedSectionIds(stripped) {
    let ids = [];
    for (const item of stripped) {
        if (item.id) {
            ids.push(`${item.id}`);
        }
        if (item.children) {
            for (const section of [item.children]) {
                const sectionIds = getStrippedSectionIds(section);
                ids = ids.concat(sectionIds);
            }
        }
    }
    return ids;
}
function getExcludedPaths(structure, strippedSectionIds) {
    let paths = [];
    for (const section of structure) {
        if (!strippedSectionIds.includes(section.id)) {
            if (section.type === PAGE) {
                paths.push(`!${section.metadata_path}`);
                paths.push(`!${section.content_path}`);
            }
            else {
                paths.push(`!${section.metadata_path}`);
                paths.push(`!${section.metadata_path}/**`);
            }
        }
        if (section.children) {
            const childrenPaths = getExcludedPaths(section.children, strippedSectionIds);
            paths = paths.concat(childrenPaths);
        }
    }
    return paths;
}
function copyStripped(srcDir, dstDir, paths) {
    return __awaiter(this, void 0, void 0, function* () {
        const mapPaths = lodash_1.default.filter(paths, _ => typeof _ != 'string');
        const stringPaths = [];
        stringPaths.push('.guides/**');
        stringPaths.push('.codio');
        stringPaths.push('.codio-menu');
        stringPaths.push('.settings');
        stringPaths.push('!.github/**');
        stringPaths.push('!**/index.json');
        lodash_1.default.forEach(lodash_1.default.filter(paths, _ => typeof _ === 'string'), path => stringPaths.push(path));
        lodash_1.default.forEach(config_1.excludePaths, path => stringPaths.push(path));
        yield (0, recursive_copy_1.default)(srcDir, dstDir, {
            filter: stringPaths,
            overwrite: true,
            dot: true
        });
        for (const map of mapPaths) {
            try {
                yield (0, recursive_copy_1.default)(path_1.default.join(srcDir, map.source), path_1.default.join(dstDir, map.destination), {
                    overwrite: true,
                    dot: true
                });
            }
            catch (_) {
                console.error(_);
            }
        }
    });
}
function updateRootMetadata(structure, metadata, dstDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const filePath = path_1.default.join(dstDir, GUIDES_CONTENT_DIR, INDEX_METADATA_FILE);
        metadata['order'] = lodash_1.default.map(structure, item => item.name);
        yield fs_1.default.promises.writeFile(filePath, JSON.stringify(metadata, undefined, ' '));
    });
}
function updateMetadata(structure, dstDir) {
    return __awaiter(this, void 0, void 0, function* () {
        for (const item of structure) {
            if (item.children) {
                const filePath = path_1.default.join(dstDir, GUIDES_CONTENT_DIR, item['section_path'], INDEX_METADATA_FILE);
                const data = {
                    id: item.id,
                    title: item.title,
                    type: item.type,
                    order: lodash_1.default.map(item.children, child => child.name)
                };
                yield fs_1.default.promises.writeFile(filePath, JSON.stringify(data, undefined, ' '));
                yield updateMetadata(item.children, dstDir);
            }
        }
    });
}
function mapToObject(map) {
    return map.size === 0 ? [] :
        Array.from(map).reduce((obj, [key, value]) => (Object.assign(obj, { [key]: value })), {});
}
exports.mapToObject = mapToObject;
function createTar(basePath, paths, excludePaths) {
    return __awaiter(this, void 0, void 0, function* () {
        const dir = yield fs_1.default.promises.mkdtemp('/tmp/codio_export');
        const file = path_1.default.join(dir, 'archive.tar');
        yield tar_1.default.c({
            file,
            cwd: basePath,
            filter: (path) => {
                if (!excludePaths) {
                    return true;
                }
                for (const exclude of excludePaths) {
                    if (lodash_1.default.startsWith(path, exclude)) {
                        return false;
                    }
                }
                return true;
            }
        }, paths);
        const zst = path_1.default.join(dir, 'archive.tar.zst');
        yield new Promise((resolve, reject) => {
            fs_1.default.createReadStream(file)
                .pipe((0, simple_zstd_1.ZSTDCompress)())
                .pipe(fs_1.default.createWriteStream(path_1.default.join(dir, 'archive.tar.zst')))
                .on('finish', resolve)
                .on('error', reject);
        });
        return zst;
    });
}
exports.createTar = createTar;
function secondsToDate(seconds) {
    const t = new Date(1970, 0, 1); // Epoch
    t.setUTCSeconds(seconds);
    return t;
}
exports.secondsToDate = secondsToDate;
function convertToGuidesV3(cwd) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('guides conversion process...');
        try {
            yield execShellCommand(`curl "https://static-assets.codio.com/guides-converter-v3/guides-converter-v3-${CONVERTER_VERSION}" --output guides-converter-v3`, cwd);
            yield execShellCommand('chmod +x ./guides-converter-v3', cwd);
            yield execShellCommand('./guides-converter-v3', cwd);
            yield execShellCommand('rm guides-converter-v3', cwd);
        }
        catch (error) {
            throw new Error(error);
        }
    });
}
exports.convertToGuidesV3 = convertToGuidesV3;
function execShellCommand(command, cwd) {
    return new Promise((resolve, reject) => {
        child_process_1.default.exec(command, { cwd }, (error, stdout, stderr) => {
            if (error) {
                reject(error);
            }
            resolve(stdout ? stdout : stderr);
        });
    });
}
function getApiV1Url() {
    return `https://octopus.${config_2.default.getDomain()}/api/v1`;
}
exports.getApiV1Url = getApiV1Url;
function getBearer() {
    if (!config_2.default) {
        throw new Error('No Config');
    }
    const token = config_2.default.getToken();
    return {
        'Authorization': `Bearer ${token}`
    };
}
exports.getBearer = getBearer;
const tools = {
    reduce,
    mapToObject,
    createTar,
    secondsToDate,
    readMetadataFile,
    getGuidesStructure
};
exports.default = tools;
